# Bio.pm -- an example of a Perl module, stuffed with some useful subs.

package Bio;

# Have this module automatically export certain symbols
# when it is 'used' or 'required' in some other code.
# The symbols exported are listed in @EXPORT.
#
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(read_fasta dna_to_rna rna_to_protein);

use strict;

# In modules we often use 'carp' and 'croak' instead
# of 'warn' and 'die', respectively, because they
# displace blame onto the module's user.  These subs
# are found in the standard module 'Carp'.
#
use Carp;

######################################################
# read_fasta -- read the first sequence from a
#               FASTA file.
#
sub read_fasta {

	# Perl stores the arguments to subroutines in the 
	# @_ array, so we fetch our's now.
	my ($file) = @_;
	
	# Then we try to open the file.  If we fail, we 
	# return 'undef' which is like NULL in C, or null
	# in Java.
	unless( open IN, "< $file" ) {
		carp "Can't open $file: $!";
		return undef;
	}
	
	# Before we read the sequence, we make sure it
	# has a description line, beginning with '>'.  That
	# way we are somewhat assured we are actually
	# reading a FASTA file.
	unless( <IN> =~ /^>/ ) {
		carp "Invalid descriptor in $file";
		return undef;
	}
	
	# Okay, so now every line which doesn't start with
	# '>' is part of the sequence.  We now build the
	# sequence one line at a time until we reach the
	# end of the file or a line beginning with '>'.
	my ($seq, $line);
	while( defined($line = <IN>) && $line !~ /^>/ ) {
		$line = lc $line;
		$line =~ s/[^a-z]//sg;
		$seq .= $line;
	}
	return $seq;
}

######################################################
# dna_to_rna -- translate a DNA sequence into a RNA
#
sub dna_to_rna {
	my ($seq) = @_;
	
	# Replace every 'a' with 'u', 'c' with 'g', etc.
	$seq =~ tr/acgt/ugca/;
	
	# Then reverse the sequence.  Voila, RNA.
	return reverse $seq;
}

######################################################
# rna_to_protein -- translate a RNA sequence into a
#                   possible protein.
#
my %codonMap;
sub rna_to_protein {
	my ($seq) = @_;
	my $protein = '';
	while( $seq =~ /(...)/g ) {
		$protein .= $codonMap{$1};
	}
	return $protein;
}

######################################################
# Stuff to do when this module is loaded.
#
while(my $line = <DATA>) {
	chomp $line;
	my @codons = split ' ', $line;
	my $residue = shift @codons;
	$codonMap{lc $_} = $residue foreach @codons;
}

# All packages should end like this, so the including
# programs know they loaded okay.
1;

######################################################
# Each line after __DATA__ can be read using the
# special filehandle 'DATA'.  Otherwise, Perl ignores
# everything from __DATA__ until the end-of-file.
#
__DATA__
Ala GCU GCC GCA GCG
Arg CGU CGC CGA CGG AGA AGG
Asn AAU AAC
Asp GAU GAC
Cys UGU UGC
Gln CAA CAG
Glu GAA GAG
Gly GGU GGC GGA GGG
His CAU CAC
Ile AUU AUC AUA
Leu UUA UUG CUU CUC CUA CUG
Lys AAA AAG
Met AUG
Phe UUU UUC
Pro CCU CCC CCA CCG
Ser UCU UCC UCA UCG AGU AGC
Thr ACU ACC ACA ACG
Trp UGG
Tyr UAU UAC
Val GUU GUC GUA GUG
... UAA UAG UGA